/* FCE Ultra - NES/Famicom Emulator
*
* Copyright notice for this file:
*  Copyright (C) 2016 Cluster
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*

	$8000-FFFF:    [.... ..CC]   Low 2 bits of CHR
	A~[..PP PPPP MHOC CCCC]

	M = Mirroring (0=Vert, 1=Horz)
	H = PRG Chip Select
	P = PRG Page Select
	O = PRG Mode (0 = 32KB, 1 = 16KB)
	C = CHR Page Select

	Example Game:
	--------------------------
	SUPER 190 IN 1 multicart

*/

#include "mapinc.h"

static uint8 EXPREGS[4];

static void Sync(void) {
	uint16 prg_addr = (EXPREGS[0] & 0x3F) | (EXPREGS[1] & 0x40);
	if (EXPREGS[1] & 0x20)
	{
		//FCEU_printf("PRG 0x8000: %08x\n", prg_addr);
		setprg16(0x8000, prg_addr);
		//FCEU_printf("PRG 0xC000: %08x\n", prg_addr);
		setprg16(0xC000, prg_addr);
	}
	else {
		//FCEU_printf("PRG 0x8000: %08x\n", prg_addr & ~1);
		setprg16(0x8000, prg_addr & ~1);
		//FCEU_printf("PRG 0x000: %08x\n", prg_addr | 1);
		setprg16(0xC000, prg_addr | 1);
	}

	uint16 chr_addr = ((EXPREGS[1] & 0x1F) << 2) | (EXPREGS[2] & 3);
	//FCEU_printf("CHR: %08x\n", chr_addr);
	setchr8(chr_addr);

	setmirror((EXPREGS[1] >> 7) ^ 1);
}

static DECLFW(K1016_Write) {
	//FCEU_printf("PRG %02x => %04x\n", V, A);
	EXPREGS[0] = (A >> 8) & 0xFF;
	EXPREGS[1] = A & 0xFF;
	EXPREGS[2] = V;
	Sync();
}

static void StateRestore(int version) {
	Sync();
}

static void K1016_Reset(void) {
	EXPREGS[0] = EXPREGS[1] = EXPREGS[2] = 0;
	Sync();
}

static void K1016_Power(void) {
	SetWriteHandler(0x8000, 0xFFFF, K1016_Write);
	SetReadHandler(0x8000, 0xFFFF, CartBR);
	Sync();
}

static void K1016_Close(void) {
}

void K1016_Init(CartInfo *info) {
	info->Power = K1016_Power;
	info->Reset = K1016_Reset;
	info->Close = K1016_Close;
	GameStateRestore = StateRestore;
	AddExState(&EXPREGS, 4, 0, "EXPR");
}
